@@ -15,7 +15,9 @@
 #include <openssl/md4.h>
 #include <openssl/md5.h>
 #include <openssl/rand.h>
+#ifndef OPENSSL_NO_RC4
 #include <openssl/rc4.h>
+#endif
 #include <openssl/ripemd.h>
 #include <openssl/sha.h>
 
@@ -34,23 +36,27 @@
 #define PASS_EVP_CTX(ctx) (&(ctx))
 #endif
 
-extern "C" {
-  #include "lua.h"
-  #include "lauxlib.h"
-}
+#include "nse_lua.h"
+#if OPENSSL_API_COMPAT >= 0x10100000L
+  /* Needed for get_random_bytes, since RAND_pseudo_bytes is gone */
+  #include <nbase.h>
+#endif
 
 #include "nse_openssl.h"
 
 typedef struct bignum_data {
   BIGNUM * bn;
+  bool should_free;
 } bignum_data_t;
 
-static int nse_pushbn( lua_State *L, BIGNUM *num )
+static int nse_pushbn( lua_State *L, BIGNUM *num)
 {
   bignum_data_t * data = (bignum_data_t *) lua_newuserdata( L, sizeof(bignum_data_t));
   luaL_getmetatable( L, "BIGNUM" );
   lua_setmetatable( L, -2 );
   data->bn = num;
+  /* Currently this is true for all uses in this file. */
+  data->should_free = true;
   return 1;
 }
 
@@ -107,6 +113,20 @@ static int l_bignum_mod_exp( lua_State *L ) /** bignum_mod_exp( BIGNUM a, BIGNUM
   return nse_pushbn(L, result);
 }
 
+static int l_bignum_div( lua_State *L ) /* bignum_div( BIGNUM a, BIGNUM d ) */
+{
+  bignum_data_t * a = (bignum_data_t *) luaL_checkudata(L, 1, "BIGNUM");
+  bignum_data_t * d = (bignum_data_t *) luaL_checkudata(L, 2, "BIGNUM");
+  BIGNUM * dv = BN_new();
+  BIGNUM * rem = BN_new();
+  BN_CTX * ctx = BN_CTX_new();
+  BN_div(dv, rem, a->bn, d->bn, ctx);
+  BN_CTX_free( ctx );
+  nse_pushbn(L, dv);
+  nse_pushbn(L, rem);
+  return 2;
+}
+
 static int l_bignum_add( lua_State *L ) /** bignum_add( BIGNUM a, BIGNUM b ) */
 {
   bignum_data_t * a = (bignum_data_t *) luaL_checkudata(L, 1, "BIGNUM");
@@ -119,14 +139,14 @@ static int l_bignum_add( lua_State *L ) /** bignum_add( BIGNUM a, BIGNUM b ) */
 static int l_bignum_num_bits( lua_State *L ) /** bignum_num_bits( BIGNUM bn ) */
 {
   bignum_data_t * userdata = (bignum_data_t *) luaL_checkudata(L, 1, "BIGNUM");
-  lua_pushnumber( L, BN_num_bits( userdata->bn) );
+  lua_pushinteger( L, BN_num_bits( userdata->bn) );
   return 1;
 }
 
 static int l_bignum_num_bytes( lua_State *L ) /** bignum_num_bytes( BIGNUM bn ) */
 {
   bignum_data_t * userdata = (bignum_data_t *) luaL_checkudata(L, 1, "BIGNUM");
-  lua_pushnumber( L, BN_num_bytes( userdata->bn) );
+  lua_pushinteger( L, BN_num_bytes( userdata->bn) );
   return 1;
 }
 
@@ -218,7 +238,9 @@ static int l_bignum_bn2hex( lua_State *L ) /** bignum_bn2hex( BIGNUM bn ) */
 static int l_bignum_free( lua_State *L ) /** bignum_free( bignum ) */
 {
   bignum_data_t * userdata = (bignum_data_t *) luaL_checkudata(L, 1, "BIGNUM");
-  BN_clear_free( userdata->bn );
+  if (userdata->should_free) {
+    BN_clear_free( userdata->bn );
+  }
   return 0;
 }
 
@@ -228,7 +250,9 @@ static int l_rand_bytes( lua_State *L ) /** rand_bytes( number bytes ) */
   unsigned char * result = (unsigned char *) malloc( len );
   if (!result) return luaL_error( L, "Couldn't allocate memory.");
 
-  RAND_bytes( result, len );
+  if (RAND_bytes( result, len ) != 1) {
+    return luaL_error(L, "Failure in RAND_bytes.");
+  }
   lua_pushlstring( L, (char *) result, len );
   free( result );
   return 1;
@@ -240,9 +264,11 @@ static int l_rand_pseudo_bytes( lua_State *L ) /** rand_pseudo_bytes( number byt
   unsigned char * result = (unsigned char *) malloc( len );
   if (!result) return luaL_error( L, "Couldn't allocate memory.");
 
-  if (RAND_bytes( result, len ) != 1) {
-    return luaL_error(L, "Failure in RAND_bytes.");
-  }
+#if OPENSSL_API_COMPAT < 0x10100000L
+  RAND_pseudo_bytes( result, len );
+#else
+  get_random_bytes( result, len );
+#endif
   lua_pushlstring( L, (char *) result, len );
   free( result );
   return 1;
@@ -510,12 +536,13 @@ static int l_DES_string_to_key(lua_State *L) /** DES_string_to_key( string data
   for( int i = 1; i < 8; i++ )
     key[i] = data[i-1] << (8-i) | data[i] >> i;
 
-  DES_set_odd_parity( &key );
+  DES_set_odd_parity( &key ); // lgtm [cpp/weak-cryptographic-algorithm]
 
   lua_pushlstring( L, (char *) key, 8 );
   return 1;
 }
 
+#ifndef OPENSSL_NO_RC4
 static int l_rc4_options (lua_State *L)
 {
   lua_pushstring(L, RC4_options());
@@ -529,7 +556,7 @@ static int l_rc4_encrypt (lua_State *L)
   const char *indata = luaL_checklstring(L, 1, &len);
   unsigned char *outdata = (unsigned char *) lua_newuserdata(L, sizeof(unsigned char)*len);
 
-  RC4(key, len, (const unsigned char *)indata, outdata);
+  RC4(key, len, (const unsigned char *)indata, outdata); // lgtm [cpp/weak-cryptographic-algorithm]
   lua_pushlstring(L, (const char *)outdata, len);
 
   return 1;
@@ -541,12 +568,13 @@ static int l_rc4 (lua_State *L)
   const char *data = luaL_checklstring(L, 1, &len);
 
   lua_newuserdata(L, sizeof(RC4_KEY));
-  RC4_set_key((RC4_KEY *)lua_touserdata(L, -1), (int)len, (const unsigned char *)data);
+  RC4_set_key((RC4_KEY *)lua_touserdata(L, -1), (int)len, (const unsigned char *)data); // lgtm [cpp/weak-cryptographic-algorithm]
 
   lua_pushcclosure(L, l_rc4_encrypt, 1);
 
   return 1;
 }
+#endif
 
 static const struct luaL_Reg bignum_methods[] = {
   { "num_bits", l_bignum_num_bits },
@@ -582,6 +610,7 @@ static const struct luaL_Reg openssllib[] = {
   { "bignum_bn2hex", l_bignum_bn2hex },
   { "bignum_add", l_bignum_add },
   { "bignum_mod_exp", l_bignum_mod_exp },
+  { "bignum_div", l_bignum_div },
   { "rand_bytes", l_rand_bytes },
   { "rand_pseudo_bytes", l_rand_pseudo_bytes },
   { "md4", l_md4 },
@@ -595,8 +624,10 @@ static const struct luaL_Reg openssllib[] = {
   { "DES_string_to_key", l_DES_string_to_key },
   { "supported_digests", l_supported_digests },
   { "supported_ciphers", l_supported_ciphers },
+#ifndef OPENSSL_NO_RC4
   { "rc4_options", l_rc4_options },
   { "rc4", l_rc4 },
+#endif
   { NULL, NULL }
 };
 
@@ -619,6 +650,9 @@ LUALIB_API int luaopen_openssl(lua_State *L) {
   // metatable.__index = metatable
   lua_pushvalue( L, -1 );
   lua_setfield( L, -2, "__index" );
+  // metatable.__tostring = bignum_bn2hex
+  lua_pushcfunction( L, l_bignum_bn2hex );
+  lua_setfield( L, -2, "__tostring" );
   // register methods
   luaL_setfuncs(L, bignum_methods, 0);
 
