@@ -3,7 +3,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2016 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2020 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -52,7 +52,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: engine_poll.c 36171 2016-08-22 19:15:13Z tudor $ */
+/* $Id: engine_poll.c 38085 2020-10-05 23:00:30Z dmiller $ */
 
 #ifndef WIN32
 /* Allow the use of POLLRDHUP, if available. */
@@ -294,10 +294,6 @@ int poll_iod_modify(struct npool *nsp, struct niod *iod, struct nevent *nse, int
     pinfo->events[sd].events |= POLL_R_FLAGS;
   if (iod->watched_events & EV_WRITE)
     pinfo->events[sd].events |= POLL_W_FLAGS;
-#ifndef WIN32
-  if (iod->watched_events & EV_EXCEPT)
-    pinfo->events[sd].events |= POLL_X_FLAGS;
-#endif
 
   return 1;
 }
@@ -403,7 +399,7 @@ static inline int get_evmask(struct npool *nsp, struct niod *nsi) {
       if (pev->revents & POLL_W_FLAGS)
         evmask |= EV_WRITE;
       if (pev->events && (pev->revents & POLL_X_FLAGS))
-        evmask |= (EV_READ | EV_WRITE | EV_EXCEPT);
+        evmask |= EV_EXCEPT;
   }
   return evmask;
 }
