@@ -4,130 +4,62 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl. This header  *
+ * summarizes some key points from the Nmap license, but is no substitute  *
+ * for the actual license text.                                            *
  *                                                                         *
- * o Integrates source code from Covered Software.                         *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem                                  *
  *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, we are happy to help.  As mentioned above, we also *
- * offer an alternative license to integrate Nmap into proprietary         *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing support and updates.  They also fund the continued         *
- * development of Nmap.  Please email sales@nmap.com for further           *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.org) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
  * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem.                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: portlist.cc 37126 2018-01-28 21:18:17Z fyodor $ */
+/* $Id: portlist.cc 38214 2021-04-29 17:52:24Z dmiller $ */
 
 
 #include "nmap.h"
@@ -184,10 +116,11 @@ void Port::freeService(bool del_service) {
 void Port::freeScriptResults(void)
 {
 #ifndef NOLUA
-  while (!scriptResults.empty()) {
-    scriptResults.front().clear();
-    scriptResults.pop_front();
-  }
+    for (ScriptResults::iterator it = scriptResults.begin();
+        it != scriptResults.end(); it++) {
+      delete (*it);
+    }
+    scriptResults.clear();
 #endif
 }
 
@@ -208,7 +141,7 @@ void Port::getNmapServiceName(char *namebuf, int buflen) const {
   if (service != NULL && service->name != NULL) {
     service_name = service->name;
   } else {
-    struct servent *service;
+    const struct servent *service;
 
     service = nmap_getservbyport(portno, IPPROTO2STR(proto));
     if (service != NULL)
@@ -326,7 +259,7 @@ void PortList::getServiceDeductions(u16 portno, int protocol, struct serviceDedu
 
   port = lookupPort(portno, protocol);
   if (port == NULL || port->service == NULL) {
-    struct servent *service;
+    const struct servent *service;
 
     /* Look up the service name. */
     *sd = serviceDeductions();
@@ -396,7 +329,7 @@ void PortList::setServiceProbeResults(u16 portno, int protocol,
     /* PROBESTATE_FINISHED_NOMATCH, PROBESTATE_EXCLUDED, PROBESTATE_INCOMPLETE.
        Just look up the service name if none is provided. */
     if (sname == NULL) {
-      struct servent *service;
+      const struct servent *service;
       service = nmap_getservbyport(portno, IPPROTO2STR(protocol));
       if (service != NULL)
         sname = service->s_name;
@@ -440,12 +373,12 @@ void PortList::setServiceProbeResults(u16 portno, int protocol,
 
 
 #ifndef NOLUA
-void PortList::addScriptResult(u16 portno, int protocol, ScriptResult& sr) {
+void PortList::addScriptResult(u16 portno, int protocol, ScriptResult *sr) {
   Port *port;
 
   port = createPort(portno, protocol);
 
-  port->scriptResults.push_back(sr);
+  port->scriptResults.insert(sr);
 }
 #endif
 
@@ -616,7 +549,7 @@ int PortList::getStateCounts(int state) const {
    except that if you ask for both TCP, UDP & SCTP, every TCP port
    will be returned before we start returning UDP and SCTP ports */
 Port *PortList::nextPort(const Port *cur, Port *next,
-                         int allowed_protocol, int allowed_state) {
+                         int allowed_protocol, int allowed_state) const {
   int proto;
   int mapped_pno;
   Port *port;
@@ -800,13 +733,20 @@ void PortList::initializePortMap(int protocol, u16 *ports, int portcount) {
    most popular one.  Returns the state if there is one, but returns
    PORT_UNKNOWN if there are no (more) states which qualify for
    consolidation */
-int PortList::nextIgnoredState(int prevstate) {
+int PortList::nextIgnoredState(int prevstate) const {
 
   int beststate = PORT_UNKNOWN;
+  int count = 0;
+  int prevcount = 0;
+  int bestcount = 0;
+
+  if (prevstate != PORT_UNKNOWN) {
+    prevcount = getStateCounts(prevstate);
+  }
 
   for(int state=0; state < PORT_HIGHEST_STATE; state++) {
     /* The state must be ignored */
-    if (!isIgnoredState(state))
+    if (!isIgnoredState(state, &count))
       continue;
 
     /* We can't give the same state again ... */
@@ -815,24 +755,28 @@ int PortList::nextIgnoredState(int prevstate) {
     /* If a previous state was given, we must have fewer ports than
        that one, or be tied but be a larger state number */
     if (prevstate != PORT_UNKNOWN &&
-        (getStateCounts(state) > getStateCounts(prevstate) ||
-         (getStateCounts(state) == getStateCounts(prevstate) && state <= prevstate)))
+        (count > prevcount ||
+         (count == prevcount && state <= prevstate)))
       continue;
 
     /* We only qualify if we have more ports than the current best */
-    if (beststate != PORT_UNKNOWN && getStateCounts(beststate) >= getStateCounts(state))
+    if (beststate != PORT_UNKNOWN && bestcount >= count)
       continue;
 
     /* Yay!  We found the best state so far ... */
     beststate = state;
+    bestcount = count;
   }
 
   return beststate;
 }
 
-/* Returns true if a state should be ignored (consolidated), false otherwise */
-bool PortList::isIgnoredState(int state) {
+/* Returns true if a state should be ignored (consolidated), false otherwise.
+ * If result is true and count is provided, it will be filled with the count of
+ * ports in that state. */
+bool PortList::isIgnoredState(int state, int *count) const {
 
+  int tmp_count = 0;
   if (o.debugging > 2)
     return false;
 
@@ -843,42 +787,47 @@ bool PortList::isIgnoredState(int state) {
   if (state == PORT_OPENFILTERED && (o.verbose > 2 || o.debugging > 2))
     return false;
 
+  tmp_count = getStateCounts(state);
+  if (count != NULL) {
+    *count = tmp_count;
+  }
   /* If openonly, we always ignore states that don't at least have open
      as a possibility. */
   if (o.openOnly() && state != PORT_OPENFILTERED && state != PORT_UNFILTERED
-      && getStateCounts(state) > 0)
+      && tmp_count > 0)
     return true;
 
   int max_per_state = 25; // Ignore states with more ports than this
   /* We will show more ports when verbosity is requested */
   if (o.verbose || o.debugging) {
     if (o.ipprotscan)
-      max_per_state *= (o.verbose + 3 * o.debugging);
+      max_per_state *= ((o.verbose + 1) + 3 * o.debugging);
     else
-      max_per_state *= (o.verbose + 20 * o.debugging);
+      max_per_state *= ((o.verbose + 1) + 20 * o.debugging);
   }
 
-  if (getStateCounts(state) > max_per_state)
+  if (tmp_count > max_per_state)
     return true;
 
   return false;
 }
 
-int PortList::numIgnoredStates() {
+int PortList::numIgnoredStates() const {
   int numstates = 0;
   for(int state=0; state < PORT_HIGHEST_STATE; state++) {
-    if (isIgnoredState(state))
+    if (isIgnoredState(state, NULL))
       numstates++;
   }
   return numstates;
 }
 
-int PortList::numIgnoredPorts() {
+int PortList::numIgnoredPorts() const {
 
   int numports = 0;
+  int tmp = 0;
   for(int state=0; state < PORT_HIGHEST_STATE; state++) {
-    if (isIgnoredState(state))
-      numports += getStateCounts(state);
+    if (isIgnoredState(state, &tmp))
+      numports += tmp;
   }
   return numports;
 }
