@@ -6,144 +6,73 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl. This header  *
+ * summarizes some key points from the Nmap license, but is no substitute  *
+ * for the actual license text.                                            *
  *                                                                         *
- * o Integrates source code from Covered Software.                         *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem                                  *
  *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.org) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
  * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem.                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: scan_engine.h 36788 2017-06-07 12:32:38Z dmiller $ */
+/* $Id: scan_engine.h 38262 2021-08-06 02:47:58Z dmiller $ */
 
 #ifndef SCAN_ENGINE_H
 #define SCAN_ENGINE_H
 
 #include "scan_lists.h"
+#include "probespec.h"
 
 #include <dnet.h>
 
 #include "timing.h"
 
-#ifndef IPPROTO_SCTP
-#include "libnetutil/netutil.h"
-#endif
-
 #include <pcap.h>
 #include <list>
 #include <vector>
@@ -151,61 +80,8 @@
 #include <algorithm>
 class Target;
 
-struct probespec_tcpdata {
-  u16 dport;
-  u8 flags;
-};
-
-struct probespec_udpdata {
-  u16 dport;
-};
-
-struct probespec_sctpdata {
-  u16 dport;
-  u8 chunktype;
-};
-
-struct probespec_icmpdata {
-  u8 type;
-  u8 code;
-};
-
-struct probespec_icmpv6data {
-  u8 type;
-  u8 code;
-};
-
-#define PS_NONE 0
-#define PS_TCP 1
-#define PS_UDP 2
-#define PS_PROTO 3
-#define PS_ICMP 4
-#define PS_ARP 5
-#define PS_CONNECTTCP 6
-#define PS_SCTP 7
-#define PS_ICMPV6 8
-#define PS_ND 9
-
-/* The size of this structure is critical, since there can be tens of
-   thousands of them stored together ... */
-typedef struct probespec {
-  /* To save space, I changed this from private enum (took 4 bytes) to
-     u8 that uses #defines above */
-  u8 type;
-  u8 proto; /* If not PS_ARP -- Protocol number ... eg IPPROTO_TCP, etc. */
-  union {
-    struct probespec_tcpdata tcp; /* If type is PS_TCP or PS_CONNECTTCP. */
-    struct probespec_udpdata udp; /* PS_UDP */
-    struct probespec_sctpdata sctp; /* PS_SCTP */
-    struct probespec_icmpdata icmp; /* PS_ICMP */
-    struct probespec_icmpv6data icmpv6; /* PS_ICMPV6 */
-    /* Nothing needed for PS_ARP, since src mac and target IP are
-       avail from target structure anyway */
-  } pd;
-} probespec;
-
 /* 3rd generation Nmap scanning function.  Handles most Nmap port scan types */
-void ultra_scan(std::vector<Target *> &Targets, struct scan_lists *ports,
+void ultra_scan(std::vector<Target *> &Targets, const struct scan_lists *ports,
                 stype scantype, struct timeout_info *to = NULL);
 
 /* Determines an ideal number of hosts to be scanned (port scan, os
@@ -217,7 +93,7 @@ void ultra_scan(std::vector<Target *> &Targets, struct scan_lists *ports,
    number of hosts scanned in parallel, though rarely to significant
    levels. */
 int determineScanGroupSize(int hosts_scanned_so_far,
-                           struct scan_lists *ports);
+                           const struct scan_lists *ports);
 
 class UltraScanInfo;
 
@@ -264,6 +140,15 @@ struct IPExtraProbeData {
   } pd;
 };
 
+union _tryno_u {
+  struct {
+  u8 isPing : 1; // Is this a ping, not a scanprobe?
+  u8 seqnum : 7; // Sequence number, 0-127
+  } fields;
+  u8 opaque;
+};
+typedef union _tryno_u tryno_t;
+
 /* At least for now, I'll just use this like a struct and access
    all the data members directly */
 class UltraProbe {
@@ -276,44 +161,18 @@ public:
      internal IPProbe.  The relevant probespec is necessary for setIP
      because pspec.type is ambiguous with just the ippacket (e.g. a
      tcp packet could be PS_PROTO or PS_TCP). */
-  void setIP(u8 *ippacket, u32 iplen, const probespec *pspec);
+  void setIP(const u8 *ippacket, u32 iplen, const probespec *pspec);
   /* Sets this UltraProbe as type UP_CONNECT, preparing to connect to given
    port number*/
   void setConnect(u16 portno);
   /* Pass an arp packet, including ethernet header. Must be 42bytes */
-  void setARP(u8 *arppkt, u32 arplen);
-  void setND(u8 *ndpkt, u32 ndlen);
+  void setARP(const u8 *arppkt, u32 arplen);
+  void setND(const u8 *ndpkt, u32 ndlen);
   // The 4 accessors below all return in HOST BYTE ORDER
   // source port used if TCP, UDP or SCTP
-  u16 sport() const {
-    switch (mypspec.proto) {
-    case IPPROTO_TCP:
-      return probes.IP.pd.tcp.sport;
-    case IPPROTO_UDP:
-      return probes.IP.pd.udp.sport;
-    case IPPROTO_SCTP:
-      return probes.IP.pd.sctp.sport;
-    default:
-      return 0;
-    }
-    /* not reached */
-  }
+  u16 sport() const;
   // destination port used if TCP, UDP or SCTP
-  u16 dport() const {
-    switch (mypspec.proto) {
-    case IPPROTO_TCP:
-      return mypspec.pd.tcp.dport;
-    case IPPROTO_UDP:
-      return mypspec.pd.udp.dport;
-    case IPPROTO_SCTP:
-      return mypspec.pd.sctp.dport;
-    default:
-      /* dport() can get called for other protos if we
-       * get ICMP responses during IP proto scans. */
-      return 0;
-    }
-    /* not reached */
-  }
+  u16 dport() const;
   u32 ipid() const {
     return probes.IP.ipid;
   }
@@ -324,7 +183,7 @@ public:
   u8 protocol() const {
     return mypspec.proto;
   }
-  ConnectProbe *CP() {
+  ConnectProbe *CP() const {
     return probes.CP;  // if type == UP_CONNECT
   }
   // Arpprobe removed because not used.
@@ -337,14 +196,13 @@ public:
     return &mypspec;
   }
 
-  /* Returns true if the given tryno and pingseq match those within this
-     probe. */
-  bool check_tryno_pingseq(unsigned int tryno, unsigned int pingseq) const {
-    return (pingseq == 0 && tryno == this->tryno) || (pingseq > 0 && pingseq == this->pingseq);
+  /* Returns true if the given tryno matches this probe. */
+  bool check_tryno(u8 tryno) const {
+    return tryno == this->tryno.opaque;
   }
 
-  u8 tryno; /* Try (retransmission) number of this probe */
-  u8 pingseq; /* 0 if this is not a scanping. Otherwise a positive ping seq#. */
+  /* tryno/pingseq, depending on what type of probe this is (ping vs scanprobe) */
+  tryno_t tryno; /* Try (retransmission) number of this probe */
   /* If true, probe is considered no longer active due to timeout, but it
      may be kept around a while, just in case a reply comes late */
   bool timedout;
@@ -355,8 +213,11 @@ public:
   struct timeval sent;
   /* Time the previous probe was sent, if this is a retransmit (tryno > 0) */
   struct timeval prevSent;
-  bool isPing() {
-    return pingseq > 0;
+  bool isPing() const {
+    return tryno.fields.isPing;
+  }
+  u8 get_tryno() const {
+    return tryno.fields.seqnum;
   }
 
 private:
@@ -403,7 +264,7 @@ public:
   ~GroupScanStats();
   void probeSent(unsigned int nbytes);
   /* Returns true if the GLOBAL system says that sending is OK. */
-  bool sendOK(struct timeval *when);
+  bool sendOK(struct timeval *when) const;
   /* Total # of probes outstanding (active) for all Hosts */
   int num_probes_active;
   UltraScanInfo *USI; /* The USI which contains this GSS.  Use for at least
@@ -468,7 +329,8 @@ public:
   Target *target; /* A copy of the Target that these stats refer to. */
   HostScanStats(Target *t, UltraScanInfo *UltraSI);
   ~HostScanStats();
-  int freshPortsLeft(); /* Returns the number of ports remaining to probe */
+  bool freshPortsLeft() const; /* Returns true if there are ports remaining to probe */
+  int numFreshPortsLeft() const; /* Returns the number of ports remaining to probe */
   int next_portidx; /* Index of the next port to probe in the relevant
                        ports array in USI.ports */
   bool sent_arp; /* Has an ARP probe been sent for the target yet? */
@@ -506,25 +368,25 @@ public:
      before considering it timed out.  Uses the host values from
      target if they are available, otherwise from gstats.  Results
      returned in MICROseconds.  */
-  unsigned long probeTimeout();
+  unsigned long probeTimeout() const;
 
   /* How long I'll wait until completely giving up on a probe.
      Timedout probes are often marked as such (and sometimes
      considered a drop), but kept in the list juts in case they come
      really late.  But after probeExpireTime(), I don't waste time
      keeping them around. Give in MICROseconds */
-  unsigned long probeExpireTime(const UltraProbe *probe);
+  unsigned long probeExpireTime(const UltraProbe *probe) const;
   /* Returns OK if sending a new probe to this host is OK (to avoid
      flooding). If when is non-NULL, fills it with the time that sending
      will be OK assuming no pending probes are resolved by responses
      (call it again if they do).  when will become now if it returns
      true. */
-  bool sendOK(struct timeval *when);
+  bool sendOK(struct timeval *when) const;
 
   /* If there are pending probe timeouts, fills in when with the time of
      the earliest one and returns true.  Otherwise returns false and
      puts now in when. */
-  bool nextTimeout(struct timeval *when);
+  bool nextTimeout(struct timeval *when) const;
   UltraScanInfo *USI; /* The USI which contains this HSS */
 
   /* Removes a probe from probes_outstanding, adjusts HSS and USS
@@ -556,7 +418,7 @@ public:
      necessary.  Note that probes on probe_bench are not included
      in this value. */
   unsigned int num_probes_waiting_retransmit;
-  unsigned int num_probes_outstanding() {
+  unsigned int num_probes_outstanding() const {
     return probes_outstanding.size();
   }
 
@@ -587,7 +449,7 @@ public:
   /* Move all members of bench to retry_stack for probe retransmission */
   void retransmitBench();
 
-  bool completed(); /* Whether or not the scan of this Target has completed */
+  bool completed() const; /* Whether or not the scan of this Target has completed */
   struct timeval completiontime; /* When this Target completed */
 
   /* This function provides the proper cwnd and ssthresh to use.  It
@@ -595,7 +457,7 @@ public:
      responses have been received for this host, may look at others in
      the group.  For CHANGING this host's timing, use the timing
      memberval instead. */
-  void getTiming(struct ultra_timing_vals *tmng);
+  void getTiming(struct ultra_timing_vals *tmng) const;
   struct ultra_timing_vals timing;
   /* The most recently received probe response time -- initialized to scan start time. */
   struct timeval lastrcvd;
@@ -615,29 +477,18 @@ public:
      appropriate.  If mayincrease is non-NULL, it is set to whether
      the allowedTryno may increase again.  If it is false, any probes
      which have reached the given limit may be dealt with. */
-  unsigned int allowedTryno(bool *capped, bool *mayincrease);
+  unsigned int allowedTryno(bool *capped, bool *mayincrease) const;
 
-
-  /* Provides the next ping sequence number.  This starts at one, goes
-   up to 255, then wraps around back to 1.  If inc is true, it is
-   incremented.  Otherwise you just get a peek at what the next one
-   will be. */
-  u8 nextPingSeq(bool inc = true) {
-    u8 ret = nxtpseq;
-    if (inc) {
-      nxtpseq++;
-      if (nxtpseq == 0)
-        nxtpseq++;
-    }
-    return ret;
+  /* Provides the next ping sequence number.  This starts at zero, goes
+   up to 127, then wraps around back to 0. */
+  u8 nextPingSeq() {
+    // Has to fit in 7 bits: tryno.fields.seqnum
+    nxtpseq = (nxtpseq + 1) % 0x80;
+    return nxtpseq;
   }
   /* This is the highest try number that has produced useful results
      (such as port status change). */
   unsigned int max_successful_tryno;
-  /* This starts as true because tryno may increase based on results, but
-     it becomes false if it becomes clear that tryno will not increase
-     further during the scan */
-  bool tryno_mayincrease;
   int ports_finished; /* The number of ports of this host that have been determined */
   int numprobes_sent; /* Number of port probes (not counting pings, but counting retransmits) sent to this host */
   /* Boost the scan delay for this host, usually because too many packet
@@ -666,28 +517,28 @@ struct ultra_scan_performance_vars : public scan_performance_vars {
 
 struct HssPredicate {
 public:
-  const int operator() (const HostScanStats *lhs, const HostScanStats *rhs) const;
+  int operator() (const HostScanStats *lhs, const HostScanStats *rhs) const;
   static struct sockaddr_storage *ss;
 };
 
 class UltraScanInfo {
 public:
   UltraScanInfo();
-  UltraScanInfo(std::vector<Target *> &Targets, struct scan_lists *pts, stype scantype) {
+  UltraScanInfo(std::vector<Target *> &Targets, const struct scan_lists *pts, stype scantype) {
     Init(Targets, pts, scantype);
   }
   ~UltraScanInfo();
   /* Must call Init if you create object with default constructor */
-  void Init(std::vector<Target *> &Targets, struct scan_lists *pts, stype scantp);
+  void Init(std::vector<Target *> &Targets, const struct scan_lists *pts, stype scantp);
 
-  unsigned int numProbesPerHost();
+  unsigned int numProbesPerHost() const;
 
   /* Consults with the group stats, and the hstats for every
      incomplete hosts to determine whether any probes may be sent.
      Returns true if they can be sent immediately.  If when is non-NULL,
      it is filled with the next possible time that probes can be sent
      (which will be now, if the function returns true */
-  bool sendOK(struct timeval *tv);
+  bool sendOK(struct timeval *tv) const;
   stype scantype;
   bool tcp_scan; /* scantype is a type of TCP scan */
   bool udp_scan;
@@ -711,7 +562,7 @@ public:
       rawprotoscan: 1;
   } ptech;
 
-  bool isRawScan();
+  bool isRawScan() const;
 
   struct timeval now; /* Updated after potentially meaningful delays.  This can
                          be used to save a call to gettimeofday() */
@@ -728,25 +579,25 @@ public:
   int removeCompletedHosts();
   /* Find a HostScanStats by its IP address in the incomplete and completed
      lists.  Returns NULL if none are found. */
-  HostScanStats *findHost(struct sockaddr_storage *ss);
+  HostScanStats *findHost(struct sockaddr_storage *ss) const;
 
-  double getCompletionFraction();
+  double getCompletionFraction() const;
 
-  unsigned int numIncompleteHosts() {
+  unsigned int numIncompleteHosts() const {
     return incompleteHosts.size();
   }
   /* Call this instead of checking for numIncompleteHosts() == 0 because it
      avoids a potential traversal of the list to find the size. */
-  bool incompleteHostsEmpty() {
+  bool incompleteHostsEmpty() const {
     return incompleteHosts.empty();
   }
-  bool numIncompleteHostsLessThan(unsigned int n);
+  bool numIncompleteHostsLessThan(unsigned int n) const;
 
-  unsigned int numInitialHosts() {
+  unsigned int numInitialHosts() const {
     return numInitialTargets;
   }
 
-  void log_overall_rates(int logt);
+  void log_overall_rates(int logt) const;
   void log_current_rates(int logt, bool update = true);
 
   /* Any function which messes with (removes elements from)
@@ -763,16 +614,48 @@ public:
 
   ScanProgressMeter *SPM;
   PacketRateMeter send_rate_meter;
-  struct scan_lists *ports;
+  const struct scan_lists *ports;
   int rawsd; /* raw socket descriptor */
   pcap_t *pd;
   eth_t *ethsd;
   u32 seqmask; /* This mask value is used to encode values in sequence
                   numbers.  It is set randomly in UltraScanInfo::Init() */
+  u16 base_port;
+
 private:
 
   unsigned int numInitialTargets;
   std::multiset<HostScanStats *, HssPredicate>::iterator nextI;
+  /* We encode per-probe information like the tryno in the source
+     port, for protocols that use ports. (Except when o.magic_port_set is
+     true--then we honor the requested source port.) The tryno is
+     encoded as offsets from base_port, a base source port number (see
+     sport_encode and sport_decode). To avoid interpreting a late response from a
+     previous invocation of ultra_scan as a response for the same port in the
+     current invocation, we increase base_port by a healthy amount designed to be
+     greater than any offset likely to be used by a probe, each time ultra_scan is
+     run.
+
+     If we don't increase the base port, then there is the risk of something like
+     the following happening:
+     1. Nmap sends an ICMP echo and a TCP ACK probe to port 80 for host discovery.
+     2. Nmap receives an ICMP echo reply and marks the host up.
+     3. Nmap sends a TCP SYN probe to port 80 for port scanning.
+     4. Nmap finally receives a delayed TCP RST in response to its earlier ACK
+     probe, and wrongly marks port 80 as closed. */
+
+  /* Base port must be chosen so that there is room to add an 8-bit value (tryno)
+   * without exceeding 16 bits. We increment modulo the largest prime number N
+   * such that 33000 + N + 256 < 65536, which ensures no overlapping cycles. */
+  // Nearest prime not exceeding 65536 - 256 - 33000:
+#define PRIME_32K 32261
+  /* Change base_port to a new number in a safe port range that is unlikely to
+     conflict with nearby past or future invocations of ultra_scan. */
+  static u16 increment_base_port() {
+    static u16 g_base_port = 33000 + get_random_uint() % PRIME_32K;
+    g_base_port = 33000 + (g_base_port - 33000 + 256) % PRIME_32K;
+    return g_base_port;
+  }
 
 };
 
